
/*****************************************************************************
* 본 예제는 ModbusRTU 무료 라이브러리를 이용하여 
* TB-485의 방사율을 읽어오는 예제 코드 입니다.	
* 본 코드에서 사용하는 라이브러리는 개인, 업체 또는 특정 목적으로의 활용에 있어서
* 무결점을 보증하지 않습니다.

* Project Name : (TB-485 시리즈) RS485 Modbus-RTU Master Code
* Version : 1.0 (2019.5.20)
* SYSTEM CLOCK : 16Mhz 
* BOARD : Arduino UNO. 5V operation 

 PORT Description

1. RX : 3            
2. TX : 5	     
3. DE/RE : 2
  온도센서모듈 입력전원은 3.3V로 하셔야 하며 포트 연결 방법은 매뉴얼을 참고하십시오.
  485 IC 없이 모듈과 아두이노를 연결하면 절대 안됩니다.
  IO 포트 및 IC 손상이 발생할 수 있습니다.

 Revision history.

1. 2019.5.20  : First version is released.

****************************************************************************/

#include <ModbusRtu.h>
#include "SoftwareSerial.h"
#define DISABLE  0
#define ENABLE   1
#define DE_RE  2
#define USING_SOFTWARESERIAL  4
#define ID  1

char _key;
uint8_t Timer1_Flag=0, Data_Print = DISABLE;
uint16_t au16data[1];

SoftwareSerial mySerial(3, 5);                      // RX 3, TX 5
Modbus master(0,USING_SOFTWARESERIAL,DE_RE);          // Modbus Master, 소프트웨어시리얼사용, DE/RE 제어 포트 2번
modbus_t telegram;                                   // Master query structure

void setup() {
  pinMode(DE_RE,OUTPUT);                              // DE_RE output 
  Serial.begin(9600);                                // use the hardware serial if you want to connect to your computer via usb cable.
  master.begin(&mySerial, 19200);                     // begin the ModBus object.
  master.setTimeOut( 20 );                            // if there is no answer in 20 ms, roll over
  
  delay(200);                                         // Wait Sensor init (minimum 200ms)
  
  Serial.println(" ");
  Serial.println("Send the 'y' key to get emissivity.");
  Serial.println(" ");
  while(1)
  {
    while(!Serial.available());  // 키값이 전송될 때 까지 대기
    _key = Serial.read();
    if (_key == 'y')
    {
        Transfer_Get_Emissivity(ID);
        break;
    }
  }  
}

void loop(){
  if(master.getState() == COM_WAITING)               
  {
   master.poll();                                    // 데이터 수신
   Data_Print = ENABLE;                              // 데이터 반복 출력 방지 플레그
  }
    if ((master.getState() == COM_IDLE) && (Data_Print == ENABLE))       // 통신 상태 : idle
    {
         if(master.getLastError() == 0 )                 // 통신에 문제 없으면 0 리턴
         { 
           Serial.print("Emissivity : ");
           Serial.println((float)au16data[0]/100.0);                     
         }
         else if(master.getLastError() == NO_REPLY)          // 응답이 없을 경우. 회로 연결에 문제가 없는지 체크하세요.
         {  
            Serial.println("No reply.");
         }
         Data_Print = DISABLE;                               // 데이터 반복 출력 방지 플레그
      }
   
}

void Transfer_Get_Emissivity(uint8_t uid)
{
  if( (uid == 0) && (uid>200) )                           // ID : 1~200 
  {
    uid = 1;                                            // 범위 오류시 ID 1로 강제 설정
  }
  
   telegram.u8id = uid;                               // slave ID (default ID = 1)
   telegram.u8fct = 4;                              // function code = 04 (Read Input Registers) 
   telegram.u16RegAdd = 800;                       // start address in slave 
   telegram.u16CoilsNo = 1;                         // number of elements to read : Only 2 support.
   telegram.au16reg = au16data;                     // pointer to a memory array in the Arduino 

   master.query( telegram );                        // send query
}
