
/*****************************************************************************
* 본 예제는 ModbusRTU 무료 라이브러리를 이용하여 
* TB-485 의 방사율을 변경하는 예제 코드 입니다.	
* 본 코드에서 사용하는 라이브러리는 개인, 업체 또는 특정 목적으로의 활용에 있어서
* 무결점을 보증하지 않습니다.
*
* Project Name : (TB-485 시리즈) RS485 Modbus-RTU Master Code
* Version : 1.0 (2019.5.20)
* SYSTEM CLOCK : 16Mhz 
* BOARD : Arduino UNO. 5V operation 

 PORT Description

1. RX : 3            
2. TX : 5	     
3. DE/RE : 2
  온도센서모듈 입력전원은 3.3V로 하셔야 하며 포트 연결 방법은 매뉴얼을 참고하십시오.
  485 IC 없이 모듈과 아두이노를 연결하면 절대 안됩니다.
  IO 포트 및 IC 손상이 발생할 수 있습니다.

 Revision history.

1. 2019.5.20  : First version is released.

****************************************************************************/

#include <ModbusRtu.h>
#include "SoftwareSerial.h"
#define DISABLE  0
#define ENABLE   1
#define DE_RE  2
#define USING_SOFTWARESERIAL  4
#define ID  1
#define SetEmissivity  97     // 10 ~ 100 사이의 양의 정수만 입력 : 97은 0.97을 뜻합니다.
                              // 방사율을 변경할 경우 반드시 전원을 리셋해야 변경이 적용됩니다.

char _key;
uint8_t Data_Print = DISABLE;
uint16_t au16data[1];

SoftwareSerial mySerial(3, 5);                      // RX 3, TX 5
Modbus master(0,USING_SOFTWARESERIAL,DE_RE);          // Modbus Master, 소프트웨어시리얼사용, DE/RE 제어 포트 2번
modbus_t telegram;                                   // Master query structure

void setup() {
  pinMode(DE_RE,OUTPUT);                              // DE_RE output 
  Serial.begin(9600);                                // use the hardware serial if you want to connect to your computer via usb cable.
  master.begin(&mySerial, 19200);                     // begin the ModBus object.
  master.setTimeOut( 30 );                            // if there is no answer in 30 ms, roll over
  
  delay(200);                                         // Wait Sensor init (minimum 200ms)
  
  Serial.println(" ");
  Serial.println("Send the 'y' key to write emissivity : "+ String((float)SetEmissivity/100));
  Serial.println(" ");
  while(1)
  {
    while(!Serial.available());  // 키값이 전송될 때 까지 대기
    _key = Serial.read();
    if (_key == 'y')
    {
        Transfer_Emissivity(ID, SetEmissivity);    // TX  : 절대 이 코드라인을 반복 실행하면 안됩니다.(loop()나 while()문 안에 사용 금지)
        break;
    }
  }
}
void loop(){
  if(master.getState() == COM_WAITING)               
  {
   master.poll();                                    // 데이터 수신
   Data_Print = ENABLE;                              // 데이터 반복 출력 방지 플레그
  }
  if ((master.getState() == COM_IDLE) && (Data_Print == ENABLE))       // 통신 상태 : idle
    {
         if(master.getLastError() == 0 )                 // 통신에 문제 없으면 0 리턴
         {
           if(SetEmissivity == au16data[0])                          // 응답 확인
           {
             Serial.println("Success!!");
             Serial.println("Reset the power.");
           }
           else
           {
             Serial.println("Failed!!");
           }
         }
         else if(master.getLastError() == NO_REPLY)          // 응답이 없을 경우. 회로 연결에 문제가 없는지 체크하세요.
         {  
            Serial.println("No reply.");
         }
         Data_Print = DISABLE;                               // 데이터 반복 출력 방지 플레그
      }
}
void Transfer_Emissivity(uint16_t uid, uint16_t emissivity)
{
  if( (uid>0) &&(uid<=200) )                           // ID : 1~200 
  {
    if( (emissivity >= 10) && (emissivity <= 100) )
    {
       au16data[0] = emissivity;
       
       telegram.u8id = uid;                             //  Slave ID
       telegram.u8fct = 6;                              // function code = 06 (Write Single Registers) 
       telegram.u16RegAdd = 800;                       // start address in slave : 800
       telegram.u16CoilsNo = 1;                         // number of elements to read : 1.
       telegram.au16reg = au16data;                     // pointer to a memory array in the Arduino 
    
       master.query( telegram );                        // send query
    }
  }
  else
  {
    Serial.println("Range over... stopped..");     
  }
   
}
